try:
    from kubernetes import client as kube_client
except ModuleNotFoundError:
    kube_client = None

from . import isddc_decorator as idec
from . import isddc_func      as ifunc


@idec.TryExceptAsync(Exception, name='InitKube')
@idec.RetryAsync('InitKube', count=10, cycle=10)
async def __init__(params: dict):
    kube_config = kube_client.Configuration()
    kube_config.host = f'{params["protocol"]}://{params["api_server"]}'
    kube_config.verify_ssl = params['verify_ssl']
    kube_config.api_key = {'authorization': 'Bearer ' + params['token'].strip()}

    api_client = kube_client.ApiClient(kube_config)
    ifunc.OverFunc(api_client.close)

    load_api: list = params['load_api'].replace(' ', '').split(',')
    kube_api = getattr(kube_client, load_api[0])(api_client)

    for api_str in load_api[1:]:
        api_obj = getattr(kube_client, api_str)(api_client)

        for method_str in dir(api_obj):
            method_obj = getattr(api_obj, method_str)

            if not method_str.startswith('_') and callable(method_obj):
                setattr(kube_api, method_str, method_obj)

    return kube_api


"""
apiVersion: v1
kind: ServiceAccount
metadata:
  name: oi
  namespace: isddc

---

apiVersion: rbac.authorization.k8s.io/v1beta1
kind: ClusterRoleBinding
metadata:
  name: oi
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: ClusterRole
  name: cluster-admin
subjects:
  - kind: ServiceAccount
    name: oi
    namespace: isddc

kubectl get secret -n isddc | grep oi | awk '{print $1}' | xargs kubectl describe secret -n kube-system | grep ^token

kubectl config view --minify | grep server
"""
