import uuid
import logging

from logging import Logger

from . import isddc_filesystem as ifs


async def __init__(params: dict) -> Logger:
    return create_logger(params)


def create_logger(params: dict) -> Logger:
    name: str = params.get('name', f'isddc-{uuid.uuid4().hex[26:]}')
    logger = Logger(name, params['level'])
    formatter = logging.Formatter(params['logfmt'], params['datefmt'])

    if 'stream' in params['output']:
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if 'file' in params['output']:
        ifs.gendir(ifs.dirname(params['handler']))
        handler = logging.FileHandler(params['handler'], encoding='UTF-8')
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    return logger


def debug(msg: str, *, logger: Logger = None):
    _log('debug', msg, logger)


def info(msg: str, *, logger: Logger = None):
    _log('info', msg, logger)


def warning(msg: str, *, logger: Logger = None):
    _log('warning', msg, logger)


def error(msg: str, *, logger: Logger = None):
    _log('error', msg, logger)


def critical(msg: str, *, logger: Logger = None):
    _log('critical', msg, logger)


def _log(level: str, msg: str, logger: Logger = None):
    getattr((logger or __default__), level)(msg)


simple: Logger = create_logger({
    'name': 'simple_logger',
    'level': 'NOTSET',
    'output': 'stream',
    'datefmt': '%F %T',
    'logfmt': '[%(asctime)s] [%(levelname)s] %(message)s'
})

sine: Logger = create_logger({
    'name': 'sine_logger',
    'level': 'NOTSET',
    'output': 'stream',
    'datefmt': '%Y-%m-%d %H:%M:%S',
    'logfmt': '[%(asctime)s] [%(module)s.%(funcName)s line:%(lineno)d] [%(levelname)s] %(message)s'
})

__default__ = simple
