try:
    from pymongo import MongoClient
    from pymongo.database import Database
    from pymongo.collection import Collection
except ModuleNotFoundError:
    MongoClient = None
    Database = None
    Collection = None

from . import isddc_decorator as idec
from . import isddc_func      as ifunc

__default__: MongoClient or Database


@idec.TryExceptAsync(Exception, name='InitMongo')
@idec.RetryAsync('InitMongo', count=10, cycle=10)
async def __init__(params: dict) -> Database or MongoClient:
    db: str = params.pop('db', None)

    client = MongoClient(**params)
    ifunc.OverFunc(client.close)

    return client[db] if db else client


def specify_collection(
        collection: str,
        *,
        db: Database or MongoClient = None
) -> Collection:
    db = db or __default__

    if isinstance(db, MongoClient):
        db_str, collection = collection.split('.')
        db: Database = db[db_str]

    return db[collection]
